@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2011 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2011 Laszlo Molnar
+   Copyright (C) 1996-2013 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2013 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -269,6 +269,25 @@ __packed_struct(Mach_twolevel_hints_command)
 #include "p_mach_enum.h"
 __packed_struct_end()
 
+template <class TMachITypes>
+__packed_struct(Mach_linkedit_data_command)
+    typedef typename TMachITypes::Word Word;
+
+    Word cmd;
+    Word cmdsize;
+    Word dataoff;  // file offset of data in __LINKEDIT segment
+    Word datasize;  // file size of data in __LINKEDIT segment
+__packed_struct_end()
+
+template <class TMachITypes>
+__packed_struct(Mach_uuid_command)
+    typedef typename TMachITypes::Word Word;
+
+    Word cmd;
+    Word cmdsize;
+    unsigned char uuid[16];
+__packed_struct_end()
+
 template <class TMachITypes>
 __packed_struct(Mach_ppc_thread_state)
     typedef typename TMachITypes::Addr Addr;
@@ -392,6 +411,8 @@ struct MachClass_32
     typedef N_Mach::Mach_segsplit_info_command<MachITypes> Mach_segsplit_info_command;
     typedef N_Mach::Mach_routines_command<MachITypes> Mach_routines_command;
     typedef N_Mach::Mach_twolevel_hints_command<MachITypes> Mach_twolevel_hints_command;
+    typedef N_Mach::Mach_linkedit_data_command<MachITypes> Mach_linkedit_data_command;
+    typedef N_Mach::Mach_uuid_command<MachITypes> Mach_uuid_command;
     typedef N_Mach::Mach_ppc_thread_state<MachITypes> Mach_ppc_thread_state;
     typedef N_Mach::Mach_i386_thread_state<MachITypes> Mach_i386_thread_state;
     typedef N_Mach::Mach_AMD64_thread_state<MachITypes> Mach_AMD64_thread_state;
@@ -428,6 +449,8 @@ struct MachClass_64
     typedef N_Mach::Mach_segsplit_info_command<MachITypes> Mach_segsplit_info_command;
     typedef N_Mach::Mach_routines_command_64<MachITypes> Mach_routines_command;
     typedef N_Mach::Mach_twolevel_hints_command<MachITypes> Mach_twolevel_hints_command;
+    typedef N_Mach::Mach_linkedit_data_command<MachITypes> Mach_linkedit_data_command;
+    typedef N_Mach::Mach_uuid_command<MachITypes> Mach_uuid_command;
 
     static void compileTimeAssertions() {
         BeLePolicy::compileTimeAssertions();
@@ -452,6 +475,8 @@ typedef MachClass_Host32::Mach_dysymtab_command Mach32_dysymtab_command;
 typedef MachClass_Host32::Mach_segsplit_info_command Mach32_segsplit_info_command;
 typedef MachClass_Host32::Mach_routines_command Mach32_routines_command;
 typedef MachClass_Host32::Mach_twolevel_hints_command Mach32_twolevel_hints_command;
+typedef MachClass_Host32::Mach_linkedit_data_command Mach32_linkedit_data_command;
+typedef MachClass_Host32::Mach_uuid_command Mach32_uuid_command;
 
 typedef MachClass_Host64::Mach_segment_command Mach64_segment_command;
 typedef MachClass_Host64::Mach_section_command Mach64_section_command;
@@ -460,6 +485,8 @@ typedef MachClass_Host64::Mach_dysymtab_command Mach64_dysymtab_command;
 typedef MachClass_Host64::Mach_segsplit_info_command Mach64_segsplit_info_command;
 typedef MachClass_Host64::Mach_routines_command Mach64_routines_command;
 typedef MachClass_Host64::Mach_twolevel_hints_command Mach64_twolevel_hints_command;
+typedef MachClass_Host64::Mach_linkedit_data_command Mach64_linkedit_data_command;
+typedef MachClass_Host64::Mach_uuid_command Mach64_uuid_command;
 
 typedef MachClass_BE32::Mach_segment_command   MachBE32_segment_command;
 typedef MachClass_BE32::Mach_section_command   MachBE32_section_command;
@@ -468,6 +495,8 @@ typedef MachClass_BE32::Mach_dysymtab_command   MachBE32_dysymtab_command;
 typedef MachClass_BE32::Mach_segsplit_info_command   MachBE32_segsplit_info_command;
 typedef MachClass_BE32::Mach_routines_command   MachBE32_routines_command;
 typedef MachClass_BE32::Mach_twolevel_hints_command   MachBE32_twolevel_hints_command;
+typedef MachClass_BE32::Mach_linkedit_data_command   MachBE32_linkedit_data_command;
+typedef MachClass_BE32::Mach_uuid_command   MachBE32_uuid_command;
 
 typedef MachClass_BE64::Mach_segment_command   MachBE64_segment_command;
 typedef MachClass_BE64::Mach_section_command   MachBE64_section_command;
@@ -476,6 +505,8 @@ typedef MachClass_BE64::Mach_dysymtab_command   MachBE64_dysymtab_command;
 typedef MachClass_BE64::Mach_segsplit_info_command   MachBE64_segsplit_info_command;
 typedef MachClass_BE64::Mach_routines_command   MachBE64_routines_command;
 typedef MachClass_BE64::Mach_twolevel_hints_command   MachBE64_twolevel_hints_command;
+typedef MachClass_BE64::Mach_linkedit_data_command   MachBE64_linkedit_data_command;
+typedef MachClass_BE64::Mach_uuid_command   MachBE64_uuid_command;
 
 typedef MachClass_LE32::Mach_segment_command   MachLE32_segment_command;
 typedef MachClass_LE32::Mach_section_command   MachLE32_section_command;
@@ -484,6 +515,8 @@ typedef MachClass_LE32::Mach_dysymtab_command   MachLE32_dysymtab_command;
 typedef MachClass_LE32::Mach_segsplit_info_command   MachLE32_segsplit_info_command;
 typedef MachClass_LE32::Mach_routines_command   MachLE32_routines_command;
 typedef MachClass_LE32::Mach_twolevel_hints_command   MachLE32_twolevel_hints_command;
+typedef MachClass_LE32::Mach_linkedit_data_command   MachLE32_linkedit_data_command;
+typedef MachClass_LE32::Mach_uuid_command   MachLE32_uuid_command;
 
 typedef MachClass_LE64::Mach_segment_command   MachLE64_segment_command;
 typedef MachClass_LE64::Mach_section_command   MachLE64_section_command;
@@ -492,6 +525,8 @@ typedef MachClass_LE64::Mach_dysymtab_command   MachLE64_dysymtab_command;
 typedef MachClass_LE64::Mach_segsplit_info_command   MachLE64_segsplit_info_command;
 typedef MachClass_LE64::Mach_routines_command   MachLE64_routines_command;
 typedef MachClass_LE64::Mach_twolevel_hints_command   MachLE64_twolevel_hints_command;
+typedef MachClass_LE64::Mach_linkedit_data_command   MachLE64_linkedit_data_command;
+typedef MachClass_LE64::Mach_uuid_command   MachLE64_uuid_command;
 
 typedef MachClass_BE32::Mach_ppc_thread_state  Mach_ppc_thread_state;
 typedef MachClass_LE32::Mach_i386_thread_state Mach_i386_thread_state;
@@ -522,6 +557,8 @@ protected:
     typedef typename MachClass::Mach_segsplit_info_command Mach_segsplit_info_command;
     typedef typename MachClass::Mach_routines_command Mach_routines_command;
     typedef typename MachClass::Mach_twolevel_hints_command Mach_twolevel_hints_command;
+    typedef typename MachClass::Mach_linkedit_data_command Mach_linkedit_data_command;
+    typedef typename MachClass::Mach_uuid_command Mach_uuid_command;
 
 public:
     PackMachBase(InputFile *, unsigned cpuid, unsigned filetype,
@@ -532,7 +569,7 @@ public:
 
     // called by the generic pack()
     virtual void pack1(OutputFile *, Filter &);  // generate executable header
-    virtual void pack2(OutputFile *, Filter &);  // append compressed data
+    virtual int  pack2(OutputFile *, Filter &);  // append compressed data
     virtual void pack3(OutputFile *, Filter &) = 0;  // append loader
     virtual void pack4(OutputFile *, Filter &) = 0;  // append PackHeader
 
@@ -575,7 +612,14 @@ protected:
     Mach_header mhdri;
 
     Mach_header mhdro;
-    Mach_segment_command segcmdo;
+    Mach_segment_command segZERO;
+    Mach_segment_command segXHDR;  // location to put eXtra headers
+    Mach_section_command secXHDR;
+    Mach_segment_command segTEXT;
+    Mach_section_command secTEXT;
+    Mach_segment_command segLINK;
+    Mach_linkedit_data_command linkitem;
+    Mach_uuid_command uuid_cmd;
 
     __packed_struct(b_info)     // 12-byte header before each compressed block
         TE32 sz_unc;  // uncompressed_size
