@@ -2,8 +2,8 @@
 
    This file is part of the UPX executable compressor.
 
-   Copyright (C) 1996-2009 Markus Franz Xaver Johannes Oberhumer
-   Copyright (C) 1996-2009 Laszlo Molnar
+   Copyright (C) 1996-2013 Markus Franz Xaver Johannes Oberhumer
+   Copyright (C) 1996-2013 Laszlo Molnar
    All Rights Reserved.
 
    UPX and the UCL library are free software; you can redistribute them
@@ -26,7 +26,7 @@
  */
 
 
-//#define WANT_STL
+//#define WANT_STL 1
 #include "conf.h"
 #include "file.h"
 #include "packer.h"
@@ -211,7 +211,7 @@ bool Packer::compress(upx_bytep i_ptr, unsigned i_len, upx_bytep o_ptr,
             cconf.conf_ucl.max_offset = opt->crp.crp_ucl.max_offset;
         if (opt->crp.crp_ucl.max_match != UINT_MAX && opt->crp.crp_ucl.max_match < cconf.conf_ucl.max_match)
             cconf.conf_ucl.max_match = opt->crp.crp_ucl.max_match;
-#if defined(WITH_NRV)
+#if (WITH_NRV)
         if (ph.level >= 7 || (ph.level >= 4 && ph.u_len >= 512*1024))
             step = 0;
 #endif
@@ -475,7 +475,7 @@ void Packer::verifyOverlappingDecompression(upx_bytep o_ptr, unsigned o_size, Fi
 
 
 /*************************************************************************
-// Find overhead for in-place decompression in an heuristic way
+// Find overhead for in-place decompression in a heuristic way
 // (using a binary search). Return 0 on error.
 //
 // To speed up things:
@@ -625,7 +625,7 @@ unsigned Packer::getRandomId() const
 #endif
     while (id == 0)
     {
-#if !defined(HAVE_GETTIMEOFDAY) || defined(__DJGPP__)
+#if !(HAVE_GETTIMEOFDAY) || defined(__DJGPP__)
         id ^= (unsigned) time(NULL);
         id ^= ((unsigned) clock()) << 12;
 #else
@@ -634,7 +634,7 @@ unsigned Packer::getRandomId() const
         id ^= (unsigned) tv.tv_sec;
         id ^= ((unsigned) tv.tv_usec) << 12;  // shift into high-bits
 #endif
-#if defined(HAVE_GETPID)
+#if (HAVE_GETPID)
         id ^= (unsigned) getpid();
 #endif
         id ^= (unsigned) fi->st.st_ino;
@@ -1311,7 +1311,8 @@ void Packer::compressWithFilters(upx_bytep i_ptr, unsigned i_len,
                                  Filter *parm_ft,
                                  const unsigned overlap_range,
                                  const upx_compress_config_t *cconf,
-                                 int filter_strategy)
+                                 int filter_strategy,
+                                 int inhibit_compression_check)
 {
     parm_ft->buf_len = f_len;
     // struct copies
@@ -1487,14 +1488,17 @@ void Packer::compressWithFilters(upx_bytep i_ptr, unsigned i_len,
     this->ph = best_ph;
     *parm_ft = best_ft;
 
-    // finally check compression ratio
-    if (best_ph.c_len + best_ph_lsize >= best_ph.u_len)
-        throwNotCompressible();
-    if (!checkCompressionRatio(best_ph.u_len, best_ph.c_len))
-        throwNotCompressible();
+    // Finally, check compression ratio.
+    // Might be inhibited when blocksize < file_size, for instance.
+    if (!inhibit_compression_check) {
+        if (best_ph.c_len + best_ph_lsize >= best_ph.u_len)
+            throwNotCompressible();
+        if (!checkCompressionRatio(best_ph.u_len, best_ph.c_len))
+            throwNotCompressible();
 
-    // postconditions 2)
-    assert(best_ph.overlap_overhead > 0);
+        // postconditions 2)
+        assert(best_ph.overlap_overhead > 0);
+    }
 
     // convenience
     buildLoader(&best_ft);
@@ -1508,10 +1512,11 @@ void Packer::compressWithFilters(upx_bytep i_ptr, unsigned i_len,
 void Packer::compressWithFilters(Filter *ft,
                                  const unsigned overlap_range,
                                  const upx_compress_config_t *cconf,
-                                 int filter_strategy)
+                                 int filter_strategy,
+                                 int inhibit_compression_check)
 {
     compressWithFilters(ft, overlap_range, cconf, filter_strategy,
-                        0, 0, 0, NULL, 0);
+                        0, 0, 0, NULL, 0, inhibit_compression_check);
 }
 
 
@@ -1522,7 +1527,8 @@ void Packer::compressWithFilters(Filter *ft,
                                  unsigned filter_off,
                                  unsigned ibuf_off,
                                  unsigned obuf_off,
-                                 const upx_bytep hdr_ptr, unsigned hdr_len)
+                                 const upx_bytep hdr_ptr, unsigned hdr_len,
+                                 int inhibit_compression_check)
 {
     ibuf.checkState(); obuf.checkState();
 
@@ -1538,7 +1544,8 @@ void Packer::compressWithFilters(Filter *ft,
                         o_ptr,
                         f_ptr, f_len,
                         hdr_ptr, hdr_len,
-                        ft, overlap_range, cconf, filter_strategy);
+                        ft, overlap_range, cconf, filter_strategy,
+                        inhibit_compression_check);
 
     ibuf.checkState(); obuf.checkState();
 }
